@@ -1,4 +1,4 @@
-/* $Id: bogoconfig.c,v 1.243 2006/08/09 23:18:21 relson Exp $ */
+/* $Id: bogoconfig.c 6993 2013-06-28 21:54:23Z m-a $ */
 
 /*****************************************************************************
 
@@ -86,19 +86,23 @@ static void display_tag_array(const char *label, FIELD *array);
 static void process_arglist(int argc, char **argv, priority_t precedence, int pass);
 static bool get_parsed_value(char **arg, double *parm);
 static void comma_parse(char opt, const char *arg, double *parm1, double *parm2, double *parm3);
+static bool token_count_conflict(void);
 
 /*---------------------------------------------------------------------------*/
 
 static struct option longopts_bogofilter[] = {
     /* longoptions.h - common options */
     LONGOPTIONS_COMMON
-    /* longoptions.h - bogofilter/-lexer options */
+    /* longoptions.h - bogofilter */
+    LONGOPTIONS_MAIN
+    LONGOPTIONS_MAIN_TUNE
+    /* longoptions.h - bogofilter/bogolexer options */
     LONGOPTIONS_LEX
     /* longoptions.h - bogofilter/bogoutil options */
     LONGOPTIONS_DB
     /* bogofilter specific options */
     { "classify-files",			N, 0, 'B' },
-    { "syslog-tag",			N, 0, 'L' },
+    { "syslog-tag",			R, 0, 'L' },
     { "classify-mbox",			N, 0, 'M' },
     { "unregister-nonspam",		N, 0, 'N' },
     { "dataframe",			N, 0, 'R' },
@@ -117,7 +121,7 @@ static struct option longopts_bogofilter[] = {
     { "debug-to-stdout",		N, 0, 'D' },
     { "no-header-tags",			N, 0, 'H' },
     { "query",				N, 0, 'Q' },
-    { "db-cachesize",			N, 0, 'k' },
+    { "db-cachesize",			R, 0, 'k' },
     { "ns-esf",				R, 0, O_NS_ESF },
     { "sp-esf",				R, 0, O_SP_ESF },
     { "ham-cutoff",			R, 0, O_HAM_CUTOFF },
@@ -129,6 +133,7 @@ static struct option longopts_bogofilter[] = {
     { "robx",				R, 0, O_ROBX },
     { "spam-cutoff",			R, 0, O_SPAM_CUTOFF },
     { "spam-header-name",		R, 0, O_SPAM_HEADER_NAME },
+    { "spam-header-place",		R, 0, O_SPAM_HEADER_PLACE },
     { "spam-subject-tag",		R, 0, O_SPAM_SUBJECT_TAG },
     { "spamicity-formats",		R, 0, O_SPAMICITY_FORMATS },
     { "spamicity-tags",			R, 0, O_SPAMICITY_TAGS },
@@ -227,7 +232,7 @@ static bool get_parsed_value(char **arg, double *parm)
     return ok;
 }
 
-void comma_parse(char opt, const char *arg, double *parm1, double *parm2, double *parm3)
+static void comma_parse(char opt, const char *arg, double *parm1, double *parm2, double *parm3)
 {
     char *parse = xstrdup(arg);
     char *copy = parse;
@@ -354,6 +359,9 @@ static const char *help_text[] = {
     "  --terse-format                    short form\n",
     "  --thresh-update                   no update if near 0 or 1\n",
     "  --timestamp                       enable/disable token timestamps\n",
+    "  --token-count                     fixed token count for scoring\n",
+    "  --token-count-min                 min token count for scoring\n",
+    "  --token-count-max                 max token count for scoring\n",
 #ifndef	DISABLE_UNICODE
     "  --unicode                         enable/disable unicode based wordlist\n",
 #endif
@@ -387,8 +395,9 @@ static void print_version(void)
     (void)fprintf(stdout,
 		  "%s version %s\n"
 		  "    Database: %s\n"
-		  "Copyright (C) 2002-2003 Eric S. Raymond, Adrian Otto, Gyepi Sam.\n"
-		  "Copyright (C) 2002-2006 David Relson, Matthias Andree, Greg Louis\n\n"
+		  "Copyright (C) 2002-2010 David Relson, Matthias Andree\n"
+		  "Copyright (C) 2002-2004 Greg Louis\n"
+		  "Copyright (C) 2002-2003 Eric S. Raymond, Adrian Otto, Gyepi Sam\n\n"
 		  "%s comes with ABSOLUTELY NO WARRANTY.  "
 		  "This is free software, and\nyou are welcome to "
 		  "redistribute it under the General Public License.  "
@@ -431,7 +440,7 @@ static void process_arglist(int argc, char **argv, priority_t precedence, int pa
 	int this_option_optind = optind ? optind : 1;
 	const char *name;
 
-	option = getopt_long(argc, argv, OPTIONS,
+	option = getopt_long_chk(argc, argv, OPTIONS,
 			     longopts_bogofilter, &option_index);
 
 	if (option == -1)
@@ -469,12 +478,17 @@ static void process_arglist(int argc, char **argv, priority_t precedence, int pa
 	    verbose = max(1, verbose); 	/* force printing */
 	    set_terse_mode_format(inv_terse_mode);
 	}
+	
+	if (token_count_conflict()) {
+	    fprintf(stderr, "Conflicting token count arguments given.\n");
+	    exit(EX_ERROR);
+	}
     }
 
     return;
 }
 
-void process_arg(int option, const char *name, const char *val, priority_t precedence, arg_pass_t pass)
+int process_arg(int option, const char *name, const char *val, priority_t precedence, arg_pass_t pass)
 {
     switch (option)
     {
@@ -518,6 +532,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	break;
 
     case 'e':
+    case O_HAM_TRUE:
 	nonspam_exits_zero = true;
 	break;
 
@@ -713,12 +728,16 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
     case O_SPAMICITY_FORMATS:		set_spamicity_formats(val);				break;
     case O_SPAMICITY_TAGS:		set_spamicity_tags(val);				break;
     case O_SPAM_HEADER_NAME:		spam_header_name = get_string(name, val);		break;
+    case O_SPAM_HEADER_PLACE:		spam_header_place = get_string(name, val);		break;
     case O_SPAM_SUBJECT_TAG:		spam_subject_tag = get_string(name, val);		break;
     case O_STATS_IN_HEADER:		stats_in_header = get_bool(name, val);			break;
     case O_TERSE:			terse = get_bool(name, val);				break;
     case O_TERSE_FORMAT:		terse_format = get_string(name, val);			break;
     case O_THRESH_UPDATE:		get_double(name, val, &thresh_update);			break;
     case O_TIMESTAMP:			timestamp_tokens = get_bool(name, val);			break;
+    case O_TOKEN_COUNT_FIX:             token_count_fix = atoi(val);                            break;
+    case O_TOKEN_COUNT_MIN:             token_count_min = atoi(val);                            break;
+    case O_TOKEN_COUNT_MAX:             token_count_max = atoi(val);                            break;
     case O_UNSURE_SUBJECT_TAG:		unsure_subject_tag = get_string(name, val);		break;
     case O_UNICODE:			encoding = get_bool(name, val) ? E_UNICODE : E_RAW;	break;
     case O_WORDLIST:			configure_wordlist(val);				break;
@@ -734,10 +753,13 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	    exit(EX_ERROR);
 	}
     }
+
+    return 0;
 }
 
 #define	Q1	if (query >= 1)
 #define	Q2	if (query >= 2)
+#define	Q3	if (query >= 3)
 
 #define YN(b) (b ? "Yes" : "No")
 #define NB(b) ((b != NULL && *b != '\0') ? b : "''")
@@ -754,6 +776,10 @@ rc_t query_config(void)
     Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "ns_esf", ns_esf, ns_esf);
     Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "sp_esf", sp_esf, sp_esf);
     Q1 fprintf(stdout, "\n");
+    Q3 fprintf(stdout, "%-17s = %d\n",    "token-count",         token_count_fix);
+    Q3 fprintf(stdout, "%-17s = %d\n",    "token-count-min",     token_count_min);
+    Q3 fprintf(stdout, "%-17s = %d\n",    "token-count-max",     token_count_max);
+    Q3 fprintf(stdout, "\n");
     Q1 fprintf(stdout, "%-17s = %s\n",    "block-on-subnets",    YN(block_on_subnets));
     Q1 fprintf(stdout, "%-17s = %s\n",    "encoding",		 (encoding != E_UNICODE) ? "raw" : "utf-8");
     Q1 fprintf(stdout, "%-17s = %s\n",    "charset-default",     charset_default);
@@ -766,6 +792,7 @@ rc_t query_config(void)
     Q2 fprintf(stdout, "%-17s = %ld\n",   "timestamp-date",      (long int)today);
     Q1 fprintf(stdout, "\n");
     Q1 fprintf(stdout, "%-17s = %s\n", "spam-header-name",    spam_header_name);
+    Q1 fprintf(stdout, "%-17s = %s\n", "spam-header-place",   NB(spam_header_place));
     Q1 fprintf(stdout, "%-17s = %s\n", "spam-subject-tag",    NB(spam_subject_tag));
     Q1 fprintf(stdout, "%-17s = %s\n", "unsure-subject-tag",  NB(unsure_subject_tag));
     Q2 fprintf(stdout, "%-18s = %s\n", "syslog-tag",          NB(logtag));
@@ -813,3 +840,20 @@ static void display_tag_array(const char *label, FIELD *array)
 	fprintf(stdout, "%s %s", (i == 0) ? "" : ",", array[i]);
     fprintf(stdout, "\n");
 }
+
+static bool token_count_conflict(void)
+{
+    if (token_count_fix != 0) {
+	if (token_count_fix < token_count_min)
+	    return true;
+    }
+
+    if (token_count_max != 0) {
+	if (token_count_max < token_count_min)
+	    return true;
+	if (token_count_max < token_count_fix)
+	    return true;
+    }
+
+    return false;
+}
