@@ -1,4 +1,4 @@
-/* $Id: lexer.c 6842 2009-07-31 09:02:41Z m-a $ */
+/* $Id: lexer.c 6988 2013-01-20 14:02:48Z m-a $ */
 
 /**
  * \file lexer.c
@@ -43,14 +43,12 @@ lexer_t *lexer = NULL;
 
 static lexer_t v3_lexer = {
     yylex,
-    &yytext,
-    &yyleng
+    lexer_v3_get_token
 };
 
 lexer_t msg_count_lexer = {
     read_msg_count_line,
-    &msg_count_text,
-    &msg_count_leng
+    msg_count_get_token
 };
 
 /* Function Prototypes */
@@ -197,7 +195,7 @@ static int get_decoded_line(buff_t *buff)
      * One very long physical line could break up into more
      * than one of these. */
 
-    if (passthrough && passmode == PASS_MEM && count > 0)
+    if (passthrough && count > 0)
 	textblock_add(linebuff->t.u.text+linebuff->read, (size_t) count);
 
     if ( !msg_header && 
